@@ -1,27 +1,30 @@
 mlogit <- function(formula, data, subset, weights = NULL, na.action, alt.subset = NULL, reflevel= NULL, ...){
+  
+  # first check whether arguments for mlogit.data are present: if so run mlogit.data
+  mf <- match.call(expand.dots = TRUE)
+  m <- match(c("data","choice","shape","varying","sep","alt.var","id.var","alt.levels","opposite"),
+             names(mf), 0L)
+  use.mlogit.data <- sum(m[-1]) > 0
+  nframe <- length(sys.calls())
+  if (use.mlogit.data){
+    mf <- mf[c(1L, m)]
+    mf[[1L]] <- as.name("mlogit.data")
+    data <- eval(mf, parent.frame())
+    new.data.name <- "mydata"
+    assign(new.data.name,data,env=sys.frame(which=nframe))
+  }
+  
   class(formula) <- c("logitform","formula")
   expanded.formula <- expand.formula(formula,data)
   y.name <- as.character(formula[[2]])
   alt.name <- names(data)[2]
   chid.name <- names(data)[1]
-  if (!is.null(alt.subset)){
-    choice <- data[[2]][data[[y.name]]]
-    choice <- choice %in% alt.subset
-    id <- unique(data[[1]])
-    names(choice) <- as.character(id)
-    id.kept <- choice[as.character(data[[1]])]
-    alt.kept <- data[[2]] %in% alt.subset
-    data <- data[id.kept & alt.kept,]
-#    data <- subset(data,id.kept & alt.kept)
-    data[[2]] <- data[[2]][,drop=TRUE]
-  }
-  if (!is.null(reflevel)){
-    data[[alt.name]] <- relevel(data[[alt.name]],reflevel)
-  }
   start.time <- proc.time()
   cl <- match.call()
   cl$formula <- formula
-  # model.frame
+
+  # model.frame (with the data provided or the one computed by mlogit.data
+  
   mf <- match.call(expand.dots = FALSE)
   m <- match(c("formula", "data", "subset", "na.action","weights"),
              names(mf), 0L)
@@ -29,20 +32,59 @@ mlogit <- function(formula, data, subset, weights = NULL, na.action, alt.subset
   mf$drop.unused.levels <- TRUE
   mf[[1L]] <- as.name("model.frame")
   mf$formula <- do.call("logitform",list(mf$formula))
-  mf <- eval(mf, parent.frame())
+  if (use.mlogit.data){
+    mf$data <- as.name(new.data.name)
+    mf <- eval(mf, sys.frame(which=nframe))
+  }
+  else{
+    mf <- eval(mf, parent.frame())
+  }
+
+  if (!is.null(reflevel)){
+    mf[[alt.name]] <- relevel(mf[[alt.name]],reflevel)
+  }
+
+  if (!is.null(alt.subset)){
+    choice <- mf[[2]][mf[[y.name]]]
+    choice <- choice %in% alt.subset
+    id <- unique(mf[[1]])
+    names(choice) <- as.character(id)
+    id.kept <- choice[as.character(mf[[1]])]
+    alt.kept <- mf[[2]] %in% alt.subset
+    mf <- mf[id.kept & alt.kept,]
+    mf[[2]] <- mf[[2]][,drop=TRUE]
+  }
+  else{
+    choice <- mf[[2]]
+  }
+
   
   # balanced the data.frame i.e. insert rows with NA when an
   # alternative is not relevant
-  alt.un <- unique(data[[alt.name]])
-  chid.un <- unique(data[[chid.name]])
+
+##   alt.un <- unique(data[[alt.name]])
+##   chid.un <- unique(data[[chid.name]])
+##   n <- length(chid.un)
+##   T <- length(alt.un)
+##   rownames(mf) <- paste(data[[chid.name]],data[[alt.name]],sep=".")
+##   all.rn <- as.character(t(outer(chid.un,alt.un,paste,sep=".")))
+##   mf <- mf[all.rn,]
+##   rownames(mf) <- all.rn
+##   mf[[1]] <- rep(chid.un,each=T)
+##   mf[[2]] <- rep(alt.un,n)
+
+  alt.un <- unique(mf[[alt.name]])
+  chid.un <- unique(mf[[chid.name]])
   n <- length(chid.un)
   T <- length(alt.un)
+  rownames(mf) <- paste(mf[[chid.name]],mf[[alt.name]],sep=".")
   all.rn <- as.character(t(outer(chid.un,alt.un,paste,sep=".")))
   mf <- mf[all.rn,]
   rownames(mf) <- all.rn
   mf[[1]] <- rep(chid.un,each=T)
   mf[[2]] <- rep(alt.un,n)
 
+  
   #suppress individuals for which no choice is made
   y <- mf[[y.name]]
   delete.id <- tapply(y,mf[[1]],sum,na.rm=TRUE)
@@ -50,23 +92,28 @@ mlogit <- function(formula, data, subset, weights = NULL, na.action, alt.subset
   mf <- mf[!(mf[[1]]%in%delete.id),]
   mf[[1]] <- mf[[1]][,drop=TRUE]
 
+  
   y <- mf[[y.name]]
   X <- model.matrix(formula,mf)
-  namesX <- colnames(X)
 
+  namesX <- colnames(X)
   if (any(names(mf)=="(weights)")) mf[["(weights)"]] <- mf[["(weights)"]]/mean(mf[["(weights)"]])
 
   alt <- mf[[alt.name]]
 #  chid <- as.character(mf[[chid.name]])
   chid <- mf[[chid.name]]
+  mframe <- mf
 
   freq <- table(alt[y])
+
   X <- split(as.data.frame(X),alt)
   X <- lapply(X,as.matrix)
   y <- split(y,alt)
-  
+
+
 #  n <- length(y[[1]])
 #  K <- ncol(X[[1]])
+
   rownames.X <- split(chid,alt)
 
   y <- lapply(y,function(x){x[is.na(x)] <- FALSE;x})
@@ -75,8 +122,22 @@ mlogit <- function(formula, data, subset, weights = NULL, na.action, alt.subset
   f <- function(param) mlogit.lnl(param, X, y, weights = NULL)
   g <- function(param) mlogit.grad(param, X, y, weights = NULL)
   h <- function(param) mlogit.hess(param, X, y, weights = NULL)
+
+  mf <- match.call(expand.dots = TRUE)
+  m <- match(c("method","print.level","iterlim","start","constPar","activePar"),
+             names(mf), 0L)
+  mf <- mf[c(1L, m)]
+  mf[[1L]] <- as.name("maxLik")
+  mf[c('logLik', 'grad', 'hess')] = c(as.name('f'),as.name('g'),as.name('h'))
+  if (is.null(mf$start)) mf$start <- rep(0,ncol(X[[1]]))
+  result <- eval(mf,sys.frame(which=nframe))
+
+#  choice <- data[[choice.name]]
+  eff <- table(alt[choice])
+  n <- sum(eff)
+  logLik0 <- sum(eff*log(eff/n))
   
-  result <- maxLik(f,g,h,start=rep(0,ncol(X[[1]])), ...)
+#  result <- maxLik(f,g,h,start=rep(0,ncol(X[[1]])))
   gradient <- g(result$estimate)
   coef <- result$estimate
   P <- mlogit.P(coef,X)
@@ -100,11 +161,11 @@ mlogit <- function(formula, data, subset, weights = NULL, na.action, alt.subset
                    message = result$message
                    )
   class(est.stat) <- "est.stat"
-  result <- list(coefficients = coef, logLik = logLik,
+  result <- list(coefficients = coef, logLik = logLik, logLik0 = logLik0,
                  hessian = hessian, gradient = gradient,
                  call = cl, est.stat = est.stat, freq = freq,
                  residuals = residuals, fitted.values = fitted.values,
-                 formula = formula, expanded.formula=expanded.formula, model= mf, index=data[1:2])
+                 formula = formula, expanded.formula=expanded.formula, model= mframe, index=mf[1:2])
   class(result) <- "mlogit"
   result
 }
@@ -143,7 +204,6 @@ mlogit.hess <- function(param, X, y, weights = NULL){
   -suml( mapply(function(x,y) crossprod(x*y,y),P,XmPX,SIMPLIFY=FALSE))
 }
 
-
 print.est.stat <- function(x, ...){
   et <- x$elaps.time[3]
   i <- x$nb.iter[1]
@@ -199,7 +259,7 @@ suml <- function(x){
   s
 }
 
-mlogit.data <- function(x, choice, shape = c("wide","long"), varying = NULL, sep = ".",
+mlogit.data <- function(data, choice, shape = c("wide","long"), varying = NULL, sep = ".",
                          alt.var = NULL, id.var = NULL, alt.levels = NULL, opposite = NULL, ...){
   if (shape=="long"){
     if (is.null(id.var)){
@@ -208,17 +268,18 @@ mlogit.data <- function(x, choice, shape = c("wide","long"), varying = NULL, sep
     }
     else{
       chid.name <- id.var
-      chid.is.variable <- ifelse(is.null(x[[id.var]]),FALSE,TRUE)
+      chid.is.variable <- ifelse(is.null(data[[id.var]]),FALSE,TRUE)
     }
     choice.name <- choice
-    choice <- x[[choice]]
+    choice <- data[[choice]]
+
     if (is.null(alt.var) && is.null(alt.levels)) stop("at least one of alt.var and alt.levels should be filled")
     
     if (!is.null(alt.levels)){
       J <- length(alt.levels)
-      n <- nrow(x)/J
+      n <- nrow(data)/J
       alt <- factor(rep(alt.levels,n),levels=alt.levels)
-      if (!is.null(alt.var) && !is.null(x[[alt.var]])){
+      if (!is.null(alt.var) && !is.null(data[[alt.var]])){
         warning(paste("variable",alt.var,"exists and will be replaced"))
         alt.is.variable <- TRUE
       }
@@ -230,57 +291,74 @@ mlogit.data <- function(x, choice, shape = c("wide","long"), varying = NULL, sep
     else{
       alt.name <- alt.var
       alt.is.variable <- TRUE
-      if (!is.factor(x[[alt.name]])) x[[alt.name]] <- factor(x[[alt.name]])
-      alt.levels <- levels(x[[alt.name]])
+      if (!is.factor(data[[alt.name]])) data[[alt.name]] <- factor(data[[alt.name]])
+      alt.levels <- levels(data[[alt.name]])
       J <- length(alt.levels)
-      alt <- x[[alt.name]]
+      alt <- data[[alt.name]]
     }
-    n <- nrow(x)/J
-    if (!chid.is.variable) choiceid <- rep(1:n,each=J) else choiceid <- x[[chid.name]]
-    if (!is.logical(x[[choice.name]])) x[[choice.name]] <- as.logical(x[[choice.name]])
-    x <- cbind(ch=choiceid,alt=alt,x)
+    n <- nrow(data)/J
+    if (!chid.is.variable) choiceid <- rep(1:n,each=J) else choiceid <- data[[chid.name]]
+
+    if (!is.logical(data[[choice.name]])){
+      if (is.factor(choice) && 'yes' %in% levels(choice)) data[[choice.name]] <- data[[choice.name]] == 'yes'
+      if (is.numeric(choice)) data[[choice.name]] <- data[[choice.name]] != 0
+    }
+    data <- cbind(ch=choiceid,alt=alt,data)
     if (alt.is.variable){
-      i <- which(names(x)==alt.name)
-      x <- x[-i]
+      i <- which(names(data)==alt.name)
+      data <- data[-i]
     }
     if (chid.is.variable){
-      i <- which(names(x)==chid.name)
-      x <- x[-i]
+      i <- which(names(data)==chid.name)
+      data <- data[-i]
     }
-    names(x)[1:2] <- c(chid.name,alt.name)
-    row.names(x) <- paste(x[[chid.name]],x[[alt.name]],sep=".")
+    names(data)[1:2] <- c(chid.name,alt.name)
+    row.names(data) <- paste(data[[chid.name]],data[[alt.name]],sep=".")
   }
 
 
   
   if (shape == "wide"){
+    class(data[[choice]]) <- 'factor'
     if (is.null(alt.var)) alt <- "alt" else alt <- alt.var
     if (is.null(id.var)) chid <- "chid" else chid <- id.var
-    x <- reshape(x,varying = varying, direction = "long", sep = sep, timevar = alt, idvar = chid,  ...)
-    x <- x[order(x[[chid]],x[[alt]]),]
-    id <- x[[chid]]
-    time <- x[[alt]]
-    idpos <- which(names(x)==chid)
-    timepos <- which(names(x)==alt)
-    x <- x[,-c(idpos,timepos)]
+    if (!is.null(varying)){
+      data <- reshape(data,varying = varying, direction = "long", sep = sep, timevar = alt, idvar = chid,  ...)
+    }
+    else{
+      id.names <- as.numeric(rownames(data))
+      nb.id <- length(id.names)
+      data[[chid]] <- id.names
+      lev.ch <- levels(data[[choice]])
+      data <- data.frame(lapply(data,rep,length(lev.ch)))
+      data[[alt]] <- rep(lev.ch,each=nb.id)
+      row.names(data) <- paste(data[[chid]],data[[alt]],sep=".")
+#      rownames(data) <- paste(outer(data[[chid]],data[[alt]],sep="."))
+    }
+    data <- data[order(data[[chid]],data[[alt]]),]
+    id <- data[[chid]]
+    time <- data[[alt]]
+    idpos <- which(names(data)==chid)
+    timepos <- which(names(data)==alt)
+    data <- data[,-c(idpos,timepos)]
     id <- as.factor(id)
     time <- as.factor(time)
-    x <- cbind(id,time,x)
-    names(x)[1:2] <- c(chid,alt)
+    data <- cbind(id,time,data)
+    names(data)[1:2] <- c(chid,alt)
     if (!is.null(alt.levels)){
-      levels(x[[choice]]) <- alt.levels
-      levels(x[[alt]]) <- alt.levels
-      row.names(x) <- paste(x[[chid]],x[[alt]],sep=".")
+      levels(data[[choice]]) <- alt.levels
+      levels(data[[alt]]) <- alt.levels
+      row.names(data) <- paste(data[[chid]],data[[alt]],sep=".")
     }
-    x[[choice]] <- x[[choice]]==x[[alt]]
+    data[[choice]] <- data[[choice]]==data[[alt]]
   }
 
   if (!is.null(opposite)){
     for (i in opposite){
-      x[[i]] <- -x[[i]]
+      data[[i]] <- -data[[i]]
     }
   }
-  x[[1]] <- factor(x[[1]])
-  x[[2]] <- factor(x[[2]])
-  x
+  data[[1]] <- factor(data[[1]])
+  data[[2]] <- factor(data[[2]])
+  data
 }
